proc POP_init {}\
{
	global CH USER
	set ct [catch {set CH [lindex [socket $USER(pop_server) 110] 0]}]
	if {$ct > 0}\
	{
		myError "Cannot open a connection with $USER(pop_server)"
		return -code return
	}
	fconfigure $CH -buffering line -translation crlf 
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Cannot open a connection with $USER(pop_server)"
		return -code return
	}
	
}

proc POP_user {}\
{
	global CH USER
	if {$USER(pop_user) == "NONE"}\
	{
		myError "No POP user specified. Please check your configurations."
		return -code return
	}
	_write $CH "user $USER(pop_user)"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Error while entering $USER(pop_user)"
		return -code return
	}
}

proc POP_pass {}\
{
	global CH USER
	_write $CH "pass $USER(pop_password)"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		return 0
	}\
	else\
	{
		return 1
	}
}	
	
proc POP_stat {}\
{
	global CH
	_write $CH "stat"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Error while reading server status"
		return -code return
	}\
	else\
	{
		return $a
	}
}

proc POP_uidl {}\
{
	global CH UIDL
	set UIDL {}
	_write $CH "uidl"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Error while reading message list"
		return -code return
	}
	while {$a != "."}\
	{
		set a [_read $CH]
		if {[ctype digit [lindex $a 0]]}\
		{
			keylset UIDL [lindex $a 0] [lrange $a 1 end]
		}
	}
}
	
proc POP_list {{num ""}}\
{
	global CH LIST
	_write $CH "list $num"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Error while reading message list"
		return -code return
	}
	while {$a != "."}\
	{
		set a [_read $CH]
		if {[ctype digit [lindex $a 0]]}\
		{
			keylset LIST [lindex $a 0].size [lindex $a 1]
		}
	}	
}

proc POP_top {num lines}\
{
	global CH LIST USER
	_write $CH "top $num $lines"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Cannot read message headers"
		return -code return
	}
	set from ""
	set to ""
	set cc ""
	set bcc ""
	set date ""
	set subject ""
	set reply ""
	set id ""
	set mimeversion ""
	set contenttype ""
	set boundary ""
	while {$a != "."}\
	{
		set a [_read $CH]
		append b "$a\n"
	}
	regsub -all "\	" $b "\ " c
	regsub -all "\n\ " $c "\ " d
	foreach a [split $d "\n"]\
	{
		if {[regexp -nocase ^From: $a]}\
		{
			set from [crange $a 6 end]
		}\
		elseif {[regexp -nocase ^To: $a]}\
		{
			set to [crange $a 4 end]
		}\
		elseif {[regexp -nocase ^Cc: $a]}\
		{
			set cc "[crange $a 4 end] $cc"
		}\
		elseif {[regexp -nocase ^Bcc: $a]}\
		{
			set bcc "[crange $a 5 end] $bcc"
		}\
		elseif {[regexp -nocase ^Subject: $a]}\
		{
			set subject [crange $a 9 end]
		}\
		elseif {[regexp -nocase ^Date: $a]}\
		{
			set date [crange $a 6 end]
		}\
		elseif {[regexp -nocase ^In-Reply-To: $a]}\
		{
			set reply [crange $a 13 end]
		}\
		elseif {[regexp -nocase ^Message-Id: $a]}\
		{
			set id [crange $a 13 end-1]
		}\
		elseif {[regexp -nocase ^MIME-Version: $a]}\
		{
			if {[regexp -nocase 1\.0 $a]}\
			{
				set mimeversion "1.0"
			}
		}\
		elseif {[regexp -nocase ^Content-Type: $a]}\
		{
			if {[regexp -nocase MULTIPART $a] || [regexp -nocase MESSAGE $a]}\
			{
				set contenttype "MULTIPART/MIXED"
				set ar [join [lrange [split $a :] 1 end] :]
				foreach pair [split $ar ;]\
				{
					if {[regexp -nocase ^BOUNDARY= $pair]}\
					{
						set boundary \
							[crange $pair 10 end-1]
						break
					}
				}
			}
		}
	}
	keylset LIST $num.from $from
	keylset LIST $num.to $to
	keylset LIST $num.cc $cc
	keylset LIST $num.bcc $bcc
	keylset LIST $num.date $date
	keylset LIST $num.reply $reply
	keylset LIST $num.subject $subject
	keylset LIST $num.id $id
	keylset LIST $num.mimeversion $mimeversion
	keylset LIST $num.contenttype $contenttype
	keylset LIST $num.boundary $boundary
	if {[file exists $USER(msgtmp)/[keylget LIST $num.id]]}\
	{
		set F [open $USER(msgtmp)/[keylget LIST $num.id]]
		keylset LIST $num [gets $F]
		close $F
	}
}
		


proc POP_retr {num}\
{
	global CH LIST USER
	if {[file exists $USER(msgtmp)/[keylget LIST $num.id]]}\
	{
		return
	}
	_write $CH "retr $num"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Cannot retrieve messages"
		return -code return
	}
	while {$a != ""}\
	{
		set a [_read $CH]
	}
	set i 0
	while {$a != "."}\
	{
		set a [_read $CH]
		if {[keylget LIST $num.boundary] != "" && \
			[regexp -- "--[keylget LIST $num.boundary]" $a]}\
		{
			incr i
			set msg$i ""
		}\
		elseif {[regexp -nocase ^Content-Type: $a] && $i>0}\
		{
			foreach pair [split $a ";"]\
			{
				if {[regexp -nocase name $pair]}\
				{
					keylset LIST $num.name$i \
						[lindex [split $pair "="] end]
				}
			}
			keylset LIST $num.contenttype$i [join [lindex \
				[split [lindex [split $a ";"] 0] ":"] end]]
		}\
		elseif {[regexp -nocase ^Content-Transfer-Encoding: $a] \
			&& $i>0}\
		{
			keylset LIST $num.transferencoding$i [crange $a 27 end]
		}\
		elseif {[regexp -nocase ^Content-ID: $a] && $i>0}\
		{
			set g [crange $a 12 end-1]
			regsub "<" $g "" h
			regsub ">" $h "" f
			regsub " " $f "" g
			keylset LIST $num.contentid$i $g
		}\
		elseif {[regexp -nocase ^Content-Disposition: $a] && $i>0}\
		{
			foreach pair [split $a ";"]\
			{
				if {[lindex [split $pair ":"] 0] == "filename"}\
				{
					keylset LIST $num.name$i \
						[lindex [split $pair ":"] 1]
					break
				}
			}
		}\
		elseif {[regexp -nocase ^Content $a] && $i>0}\
		{
		}\
		else\
		{
			append msg$i "$a\n"
		}
	}
	if {$i == 0}\
	{
		keylset LIST $num.contentid0 "[keylget LIST $num.id]-0"
		write_file $USER(read-tmp)/[keylget LIST $num.contentid0]-D [expr $[subst msg0]]
	}
	for {set j 0} {$j < $i} {incr j}\
	{
		if {! [keylget LIST $num.contenttype$j {}]}\
		{
			keylset LIST $num.contenttype$j "TEXT/PLAIN"
		}
		if {[keylget LIST $num.contentid$j {}]}\
		{
			write_file $USER(read-tmp)/[keylget LIST \
				$num.contentid$j] [expr $[subst msg$j]]
		}\
		else\
		{
			keylset LIST $num.contentid$j "[keylget LIST $num.id]-$j"
			write_file $USER(read-tmp)/[keylget LIST $num.contentid$j]-D [expr $[subst msg$j]]
		}
	}
	keylset LIST $num.nbparts $i
}

proc POP_dele {num}\
{
	global CH
	_write $CH "dele $num"
	set a [_read $CH]
	if {[crange $a 0 3] == "-ERR"}\
	{
		myError "Cannot delete message"
		return -code return
	}
}
	

proc POP_noop {}\
{
	global CH
	_write $CH "noop"
	set a [_read $CH]
}
	

proc POP_quit {}\
{
	global CH 
	_write $CH "quit"
	catch {close $CH}
}
